@@ -7,55 +7,93 @@ either "compiled down" into AFDKO feature code or directly written to the
 GSUB/GPOS tables of a font.
 
 FontFeatures aims to marshal data between OTF binary representations,
-AFDKO feature code, FontDame, and a new language called the FEE (Font
-Engineering with Extensibility) language.
+AFDKO feature code, FontDame, and can power other representations such as the
+FEZ language (see the 'fez' library).
 
 A FontFeatures representation of a font will make use of two other top-level
 concepts: Features and Routines. Routines are collections of rules; they play
 a similar function to the AFDKO concept of a lookup, but unlike lookups,
 Routines do not need to be comprised of rules of the same type. You can think
 of them as functions that are called on a glyph string.
+
+Here is an example of constructing a simple feature file using fontFeatures::
+
+    ff = FontFeatures()
+
+    liga_ffi = Substitution( [ ["f"], ["f"], ["i"] ], replacement=[["f_f_i"]] )
+    liga_ffl = Substitution( [ ["f"], ["f"], ["l"] ], replacement=[["f_f_l"]] )
+    liga_fi = Substitution( [ ["f"], ["i"] ], replacement=[["fi"]] )
+    liga_ff = Substitution( [ ["f"], ["f"] ], replacement=[["f_f"]] )
+    liga_routine = Routine(rules=[liga_ffi, liga_ffl, liga_fi, liga_ff])
+
+    ff.addFeature("liga", [liga_routine])
+
+    # Export Adobe syntax
+    print(ff.asFea())
+
+    font = TTFont("Test.ttf")
+    ff.buildBinaryFeatures(font)
+    font.save("Test-liga.ttf")
 """
 
-from fontTools.ttLib import TTFont
 from collections import OrderedDict, namedtuple
 from fontTools.feaLib.ast import ValueRecord as feaLibValueRecord
 from itertools import chain
-from bidict import bidict
 from copy import copy
+from .variableScalar import VariableScalar
 
 
 class FontFeatures:
-    """An object representing the layout rules in a font."""
+    """An object representing the layout rules in a font.
+
+    The initializer has no parameters."""
 
     def __init__(self):
-        self.namedClasses = bidict({})
-        self.routines = []
-        self.features = OrderedDict()
-        self.anchors = {}
+        self.namedClasses = {}  #: A mapping of named classes to a list of glyph names which make up the class.
+        self.routines = []  #: All of the layout routines used in this font.
+        self.features = OrderedDict()  #: An ordered dictionary mapping feature tags to a list of routine references.
+        self.anchors = {}  #: A dictionary mapping glyph names to a dictionary of anchor names / positions.
         self.symbols = {}
-        self.glyphclasses = {}  # Glyph name -> category
-        self.scratch = {}  # Space for items to communicate context to each other. :(
+        self.glyphclasses = {}  #: A dictionary mapping glyph names to their categories.
+        self.scratch = {}  #: Space for items to communicate context to each other.
         self.doneUsageMarking = False
 
     def __add__(self, other):
+        """Combine two FontFeatures objects together."""
         combined = FontFeatures()
-        for k in ["namedClasses", "routines", "features", "anchors", "symbols"]:
-            if any([isinstance(getattr(combined, k), c) for c in [dict, bidict]]):
+        for k in ["namedClasses", "routines", "features", "anchors", "symbols", "glyphclasses"]:
+            if isinstance(getattr(combined, k), dict):
                 getattr(combined, k).update(getattr(self, k))
                 getattr(combined, k).update(getattr(other, k))
             else:
                 setattr(combined, k, getattr(self, k) + getattr(other, k))
-        for mine, theirs in zip(self.glyphclasses, other.glyphclasses):
-            mine |= theirs
         return combined
 
     def gensym(self, category):
+        """Generate a new unique symbol (used for labeling unlabeled data).
+
+        Args:
+            category (str): The category for this symbol
+
+        Returns: a string representing a unique label."""
         if not category in self.symbols:
             self.symbols[category] = 0
         self.symbols[category] += 1
         return f"{category}{self.symbols[category]}"
 
+    def routineNamed(self, name):
+        """Finds a routine with the given name.
+
+        Args:
+            name (str): The name to find
+
+        Returns: a :py:class:`Routine` object if the named routine was found
+          in the features object. Raises a ``ValueError`` if not."""
+        for r in self.routines:
+            if r.name == name:
+                return r
+        raise ValueError("Can't find routine '%s'" % name)
+
     def referenceRoutine(self, r):
         """Store a routine and return a reference to it.
 
@@ -83,9 +121,10 @@ class FontFeatures:
             class will be returned. If not, then the class will be stored
             and the name provided as the ``name`` argument will be returned.
         """
-        if tuple(glyphs) in self.namedClasses.inverse:
-            return self.namedClasses.inverse[tuple(glyphs)]
-        self.namedClasses[name] = tuple(glyphs)
+        for k,v in self.namedClasses.items():
+            if sorted(glyphs) == sorted(v):
+                return k
+        self.namedClasses[name] = glyphs
         return name
 
     def addFeature(self, name, rs):
@@ -103,18 +142,6 @@ class FontFeatures:
                 r = self.referenceRoutine(r)
             self.features[name].append(r)
 
-    def allRoutines(self):
-        """Return all Routines in the font.
-
-        Returns:
-            Routines stored in the preamble and within features.
-        """
-        routines = set(self.routines)
-        for k, v in self.features.items():
-            for n in v:
-                if isinstance(n, Routine):
-                    routines.add(n)
-        return list(routines)
 
     def allRules(self, ruletype=None):
         """Return all rules in the font, optionally filtered by type
@@ -128,12 +155,8 @@ class FontFeatures:
         """
 
         rules = []
-        for r in self.allRoutines():
+        for r in self.routines:
             rules.extend(r.rules)
-        for k, v in self.features.items():
-            for n in v:
-                if isinstance(n, Routine):
-                    rules.extend(n.rules)
 
         if ruletype:
             rules = filter(lambda x: isinstance(x, ruletype), rules)
@@ -148,7 +171,7 @@ class FontFeatures:
         """
         if self.doneUsageMarking:
             return
-        for r in self.allRoutines():
+        for r in self.routines:
             r.usedin = set()
         for chain in self.allRules(Chaining):
             for routinelist in chain.lookups:
@@ -183,6 +206,10 @@ class FontFeatures:
             if k.languages:
                 for l in k.languages:
                     add_language(l)
+            else:
+                for r in k.rules:
+                    for l in (r.languages or []):
+                        add_language(l)
 
         # if count > 0 and not "DFLT" in scripts:
         #     scripts["DFLT"] = []
@@ -193,10 +220,23 @@ class FontFeatures:
         self.scripts_and_languages = scripts
 
     def hasScriptSupport(self, script):
+        """Check if the features object has support for a particular script.
+
+        Args:
+            script (str): A four-character OpenType script code.
+
+        Returns: boolean
+        """
         self.hoist_languages()
         return script in self.scripts_and_languages
 
     def resolveAllRoutines(self):
+        """Resolve reference use in chains.
+
+        Checks that all routines referenced in chain rules can actually
+        be found within the object, and adds pointers to match named routine
+        references with the relevant :py:class:`Routine` object.
+        """
         for routine in self.routines:
             for r in routine.rules:
                 if not isinstance(r, Chaining):
@@ -207,6 +247,10 @@ class FontFeatures:
                     )
 
     def ensureLookupsAreReferences(self, lookuplist):
+        """Ensures that all references are lookups.
+
+        Naughty people might put :py:class:`Routine` objects directly into
+        :py:class:`Chain` lookups. This tidies them up."""
         rv = []
         if not lookuplist:
             return None
@@ -220,10 +264,28 @@ class FontFeatures:
         return rv
 
     def setGlyphClassesFromFont(self, font):
+        """Loads glyph classes from the font."""
         for g in font.exportedGlyphs():
-            self.glyphclasses[g] = font[g].category
+            if hasattr(font, "glyphs"):
+                self.glyphclasses[g] = font.glyphs[g].category
+            else:
+                self.glyphclasses[g] = font[g].category
 
     def partitionRoutine(self, routine, factor):
+        """Splits a routine based on a predicate.
+
+        This method applies a function to each rule in the routine and creates
+        distinct routines, each containing rules with the same return value
+        from the function. This is useful, for example, when exporting to
+        OpenType, to ensure that all rules in a routine must have the same type,
+        same flags, etc.
+
+        Args:
+            routine: A :py:class:`Routine` object.
+            factor: A function applied to each of the :py:class:`Rule` objects.
+
+        Returns: A list of :py:class:`Routine` objects. Additionally, modifies
+        the ``.routines`` list of the FontFeatures object."""
         if not routine.rules:
             return
         self.doneUsageMarking = False
@@ -280,7 +342,8 @@ class FontFeatures:
         return allroutines
 
     from .feaLib.FontFeatures import asFea, asFeaAST
-    from .ttLib.FontFeatures import buildGPOSGSUB
+    from .xmlLib.FontFeatures import toXML, fromXML
+    from .ttLib.FontFeatures import buildBinaryFeatures
 
 
 class Routine:
@@ -345,6 +408,9 @@ class Routine:
 
     @property
     def stage(self):
+        """Returns which shaping stage this routine is used in.
+
+        Returns: ``sub`` for substitution stage, ``pos`` for positioning stage."""
         for r in self.rules:
             if isinstance(r, Substitution):
                 return "sub"
@@ -355,6 +421,15 @@ class Routine:
             if isinstance(r, Chaining):
                 return r.stage
 
+    @property
+    def dependencies(self):
+        """Returns a list of :py:class:`Routine` objects called as lookups in
+        this Routine."""
+        deps = []
+        for r in self.rules:
+            deps.extend(r.dependencies)
+        return deps
+
     from .feaLib.Routine import asFea, asFeaAST, feaPreamble
     from .shaperLib.Routine import apply_to_buffer
     from .xmlLib.Routine import toXML, fromXML
@@ -362,6 +437,7 @@ class Routine:
 
 
 class ExtensionRoutine(Routine):
+    """OpenType-specific concept: A routine which contains other routines."""
     def __init__(self, **kwargs):
         if "routines" in kwargs:
             self.routines = kwargs["routines"]
@@ -369,10 +445,20 @@ class ExtensionRoutine(Routine):
         super().__init__(**kwargs)
 
     def apply_to_buffer(self, buf, stage=None, feature=None):
+        """Applies shaping rules from this routine to a buffer.
+
+        Args:
+            buf: A :py:class:`fontFeatures.shaperLib.Buffer` object.
+            stage (str): Shaping stage - ``sub`` or ``pos``.
+            feature (str): The feature being processed. (For debugging.)
+
+        Modifies the ``buf`` object.
+        """
         for r in self.routines:
             r.apply_to_buffer(buf, stage, feature)
 
     def asFeaAST(self):
+        """Returns this extension routine as ``fontTools.feaLib.ast`` objects."""
         import fontTools.feaLib.ast as feaast
 
         f = feaast.Block()
@@ -382,18 +468,30 @@ class ExtensionRoutine(Routine):
 
     @property
     def stage(self):
+        """Returns which shaping stage this routine is used in.
+
+        Returns: ``sub`` for substitution stage, ``pos`` for positioning stage."""
         return self.routines[0].stage
 
     @property
     def rules(self):
+        """All rules under this extension.
+
+        Returns: A flattened list of :py:class:`Rule` objects."""
         return list(chain(*[routine.rules for routine in self.routines]))
 
     @rules.setter
     def rules(self, foo):
+        """Does nothing. Don't set rules here."""
         pass
 
 
 class RoutineReference:
+    """A reference to a Routine object, used in a lookup.
+
+    Routines can be referenced either by name (for example, when loaded from a
+    textual representation), in which case they will be resolved at a later time,
+    or by providing a pointer to the :py:class:`Routine` object."""
     def __init__(self, name=None, routine=None):
         self.routine = routine
         if self.routine:
@@ -402,11 +500,13 @@ class RoutineReference:
             self.name = name
 
     def resolve(self, ff):
+        """Resolves the reference in the context of a :py:class:`FontFeatures`
+        object.
+
+        Raises a ``ValueError`` if a named routine cannot be found.
+        """
         if not self.routine:
-            for r in ff.routines:
-                if r.name == self.name:
-                    self.routine = r
-                    break
+            self.routine = ff.routineNamed(self.name)
             if not self.routine:
                 raise ValueError("Could not resolve routine")
         if not self.name:
@@ -414,15 +514,22 @@ class RoutineReference:
 
     @property
     def stage(self):
+        """Returns which shaping stage this routine is used in.
+
+        Returns: ``sub`` for substitution stage, ``pos`` for positioning stage."""
         if not self.routine:
             raise ValueError("Routine reference not resolved")
         return self.routine.stage
 
     from .xmlLib.RoutineReference import fromXML, toXML
-    from .feaLib.RoutineReference import asFea, asFeaAST, feaPreamble
+    from .feaLib.RoutineReference import asFeaAST, feaPreamble
 
+    def asFea(self):
+        """Returns this Rule as a string of AFDKO feature text."""
+        return self.asFeaAST().asFea()
 
 class Rule:
+    """A base class for all rules."""
     def asFea(self):
         """Returns this Rule as a string of AFDKO feature text."""
         return self.asFeaAST().asFea()
@@ -436,8 +543,15 @@ class Rule:
 
     @property
     def has_context(self):
+        """Does this rule have any pre- or post-context defined?"""
         return len(self.precontext) or len(self.postcontext)
 
+    @property
+    def dependencies(self):
+        """Returns a list of :py:class:`Routine` objects called as lookups in
+        this Routine."""
+        return []
+
 
 class Substitution(Rule):
     """Represents a Substitution rule.
@@ -461,6 +575,22 @@ class Substitution(Rule):
             the inner list containing Routines to apply.
         reverse: Boolean representing if the substitutions should take place from
             the end of the string.
+        force_alt: Force this substitution to be interpreted as an alternate
+            substitution.
+
+    Examples::
+
+        lig = Substitution(
+            [ ["f"], ["i"] ],
+            ["f_i"]
+        ) # sub f i by f_i;
+
+        contextual = Substitution(
+            [ ["dotbelow"] ],
+            [ ["dotbelow.post"] ],
+            precontext = [["ra-myanmar", "ra-myanmar.bt1", "ra-myanmar.bt2"]]
+        ) # sub [ra-myanmar ra-myanmar.bt1 ra-myanmar.bt2] dotbelow-myanmar'
+          # by dotbelow-myanmar.post;
     """
 
     def __init__(
@@ -474,6 +604,7 @@ class Substitution(Rule):
         lookups=None,
         reverse=False,
         flags=0,
+        force_alt=False
     ):
         self.precontext = precontext or []
         self.postcontext = postcontext or []
@@ -485,9 +616,11 @@ class Substitution(Rule):
         self.flags = flags
         self.reverse = reverse
         self.stage = "sub"
+        self.force_alt = force_alt
 
     @property
     def involved_glyphs(self):
+        """Returns a set of all glyphs involved in this rule."""
         i = set(chain.from_iterable(self.input))
         o = set(chain.from_iterable(self.replacement))
         b = set(chain.from_iterable(self.precontext))
@@ -517,6 +650,18 @@ class Chaining(Rule):
         lookups: A list of list of lookups to be applied to the glyph sequence.
             The outer list represents the positions in the input sequence, with
             the inner list containing Routines to apply.
+
+    Example::
+
+        sub_Qu = Routine(rules=[
+            Substitute([["Q"]], [["Q.beforeu"]])
+        ])
+
+        chain = Chain(
+            [["Q"]],
+            postcontext = [ ["u", "v", "u.sc", "v.sc"] ],
+            lookups = [ [sub_Qu] ]
+        ) # sub Q' lookup sub_Qu [u v u.sc v.sc];
     """
 
     def __init__(
@@ -539,6 +684,9 @@ class Chaining(Rule):
 
     @property
     def stage(self):
+        """Returns which shaping stage this routine is used in.
+
+        Returns: ``sub`` for substitution stage, ``pos`` for positioning stage."""
         for l in self.lookups:
             if not l:
                 continue
@@ -549,78 +697,52 @@ class Chaining(Rule):
 
     @property
     def involved_glyphs(self):
+        """Returns a set of all glyphs involved in this rule."""
         i = set(chain.from_iterable(self.input))
         b = set(chain.from_iterable(self.precontext))
         a = set(chain.from_iterable(self.postcontext))
         return i | b | a
 
+    @property
+    def dependencies(self):
+        """Returns a list of :py:class:`Routine` objects called as lookups in
+        this Routine."""
+        deps = []
+        for l in self.lookups:
+            for aLookup in (l or []):
+                if isinstance(aLookup, RoutineReference):
+                    deps.append(aLookup.routine)
+                else:
+                    deps.append(aLookup)
+        return deps
+
     from .feaLib.Chaining import asFeaAST, feaPreamble
     from .shaperLib.Chaining import shaper_inputs, _do_apply
     from .xmlLib.Chaining import _toXML, fromXML
+    from .ttLib.Chaining import lookup_type
 
 
 class ValueRecord(feaLibValueRecord):
-    """An extension of the fontTools.feaLib ValueRecord representation to support
-    variable font item variation stores."""
+    """A value record for representing positional changes in advance and placement.
 
-    # Actually until we have a binary compiler, we're just going to cheat and store
-    # a value record for each master.
-    def set_value_for_master(self, vf, master, vr):
-        """Set a master-specific value record.
+    See :py:class:`fontTools.feaLib.ValueRecord`, from which this inherits.
+    """
+    from .ttLib.ValueRecord import toOTValueRecord
 
-        Args:
-            vf: a babelfont.VariableFont instance
-            master: name of the master to set
-            vr: a ``ValueRecord`` specific to this master.
-        """
-        if not hasattr(self, "masters"):
-            self.masters = {}
-            # Clone this value record to all masters
-            for mastername in vf.masters.keys():
-                self.masters[mastername] = ValueRecord(
-                    xPlacement=vr.xPlacement,
-                    yPlacement=vr.yPlacement,
-                    xAdvance=vr.xAdvance,
-                    yAdvance=vr.yAdvance,
-                )
-        else:
-            self.masters[master] = vr
-
-    def get_value_for_master(self, vf, master):
-        """Get a master-specific value record.
-
-        Args:
-            vf: a ``babelfont.VariableFont`` instance
-            master: name of the master to get
-        """
-        if not hasattr(self, "masters"):
-            return self
-        return self.masters[master]
-
-    def get_value_for_location(self, vf, location):
-        """Get an interpolated value record for a particular location.
-
-        Args:
-            vf: a ``babelfont.VariableFont`` instance
-            location: A dictionary mapping axis names to user space coordinates.
-        """
-        if not hasattr(self, "masters"):
-            return self
-        locs = {
-            k: (v.xPlacement, v.yPlacement, v.xAdvance, v.yAdvance)
-            for k, v in self.masters.items()
-        }
-        values = vf.interpolate_tuples(locs, location)
-        xPlacement, yPlacement, xAdvance, yAdvance = values
-        return ValueRecord(
-            xPlacement=xPlacement,
-            yPlacement=yPlacement,
-            xAdvance=xAdvance,
-            yAdvance=yAdvance,
+    @property
+    def is_variable(self):
+        """Returns true if any of the elements of the value record are a
+        :py:class:`fontFeatures.variableScalar.VariableScalar`."""
+        return any(
+            isinstance(x, VariableScalar)
+            for x in [
+                self.xPlacement,
+                self.yPlacement,
+                self.xAdvance,
+                self.yAdvance,
+            ]
         )
 
-    from .ttLib.ValueRecord import toOTLookup
-
 
 class Positioning(Rule):
     """Represents a Positioning rule.
@@ -635,6 +757,21 @@ class Positioning(Rule):
             sequence.
         postcontext: A list of list of glyphs which must appear before the input
             sequence.
+
+    Example::
+
+        open_up_behs = Positioning(
+            [
+                ["BEi1", "BEi2"],
+                ["sda", "sdb", "dda", "ddb"]
+            ],
+            [
+                ValueRecord(xAdvance=200),
+                ValueRecord(xPlacement=50),
+            ]
+            postcontext = [ medis_finas ]
+        )
+        # pos [BEi1 BEi2]' <0 0 200 0> [sda sdb dda ddb]' <0 50 0 0> @medis_finas;
     """
 
     def __init__(
@@ -659,6 +796,7 @@ class Positioning(Rule):
 
     @property
     def involved_glyphs(self):
+        """Returns a set of all glyphs involved in this rule."""
         i = set(chain.from_iterable(self.glyphs))
         b = set(chain.from_iterable(self.precontext))
         a = set(chain.from_iterable(self.postcontext))
@@ -683,6 +821,33 @@ class Attachment(Rule):
         marks: Dictionary. They keys are names of glyphs to act as marks;
             the associated values are a two-element tuple with the coordinates
             of the anchor.
+        force_markmark: boolean. If true, force this to be interpreted as a
+            mark-to-mark operation
+
+    Whether this is a mark-to-base or mark-to-mark operation will be determined
+    by the glyph category of the glyphs involved in the `bases` dictionary and
+    the value of the `force_markmark` argument.
+
+    Examples::
+
+        ff.anchors = {
+            "a": { "top": (250, 603) },
+            "acutecomb": { "_top": (56, 0) }
+        }
+
+        top_bases = {}
+        top_marks = {}
+        for glyphname, anchors in ff.anchors.items():
+            for anchorname, position in anchors.items():
+                if anchorname == "top":
+                    top_bases[glyphname] = position
+                if anchorname == "_top":
+                    top_marks[glyphname] = position
+
+        # top_bases = { "a": (260,603) }
+        # top_marks = { "acutecomb": (56,0) }
+
+        tops = Attachment("top", "_top", top_bases, top_marks)
     """
 
     def __init__(
@@ -694,6 +859,8 @@ class Attachment(Rule):
         flags=0,
         address=None,
         font=None,
+        languages=None,
+        force_markmark=False
     ):
         self.base_name = base_name
         self.mark_name = mark_name
@@ -703,17 +870,22 @@ class Attachment(Rule):
         self.address = address
         self.font = font
         self.stage = "pos"
+        self.languages = languages or []
+        self.force_markmark = force_markmark
 
     @property
     def is_cursive(self):
+        """Returns true if this is a cursive attachment rule."""
         return self.base_name == "cursive_entry" or self.base_name == "entry"  # XXX
 
     from .feaLib.Attachment import asFeaAST, feaPreamble
     from .shaperLib.Attachment import shaper_inputs, _do_apply, would_apply_at_position
     from .xmlLib.Attachment import _toXML, fromXML
+    from .ttLib.Attachment import lookup_type
 
     @property
     def involved_glyphs(self):
+        """Returns a set of all glyphs involved in this rule."""
         b = set(self.bases.keys())
         m = set(self.marks.keys())
         return b | m
