@@ -2,9 +2,10 @@
 import fontTools.feaLib.ast as feaast
 from glyphtools import categorize_glyph
 import warnings
+from fontFeatures.variableScalar import VariableScalar
 
 
-def glyphref(g):
+def _glyphref(g):
     if len(g) == 1:
         return feaast.GlyphName(g[0])
     return feaast.GlyphClass([feaast.GlyphName(x) for x in g])
@@ -39,7 +40,7 @@ def feaPreamble(self, ff):
                 feaast.MarkClassDefinition(
                     feaast.MarkClass(self.base_name),
                     feaast.Anchor(*mark[1]),
-                    glyphref(mark[0]),
+                    _glyphref(mark[0]),
                 )
             )
             ff.scratch["mark_classes_done"][(self.base_name, tuple(mark[0]))] = True
@@ -48,12 +49,17 @@ def feaPreamble(self, ff):
 
 def asFeaAST(self):
     b = feaast.Block()
+    # if any(
+    #     isinstance(x[0], VariableScalar) or isinstance(x[1], VariableScalar)
+    #     for x in list(self.bases.values()) + list(self.marks.values())
+    # ):
+    #     raise ValueError("Can't directly express a variable anchor in FEA")
     if self.is_cursive:
         allglyphs = set(self.bases.keys()) | set(self.marks.keys())
         for g in allglyphs:
             b.statements.append(
                 feaast.CursivePosStatement(
-                    glyphref([g]),
+                    _glyphref([g]),
                     g in self.bases and feaast.Anchor(*self.bases[g]),
                     g in self.marks and feaast.Anchor(*self.marks[g]),
                 )
@@ -64,11 +70,13 @@ def asFeaAST(self):
         for base in self.baseslist:
             statementtype = feaast.MarkBasePosStatement
             if self.font:
-                if categorize_glyph(self.font,base[0][0])[0] == "mark":
+                if categorize_glyph(self.font, base[0][0])[0] == "mark":
                     statementtype = feaast.MarkMarkPosStatement
+            if self.force_markmark:
+                statementtype = feaast.MarkMarkPosStatement
             b.statements.append(
                 statementtype(
-                    glyphref(base[0]),
+                    _glyphref(base[0]),
                     [[feaast.Anchor(*base[1]), feaast.MarkClass(self.base_name)]],
                 )
             )
