@@ -1,4 +1,4 @@
-# Code for converting a Routine object into feaLib statements
+"""Routines for converting RoutineReference objects to and from Adobe FEA."""
 import fontTools.feaLib.ast as feaast
 from fontFeatures.ttLib.Substitution import lookup_type as sub_lookup_type
 from fontFeatures.ttLib.Positioning import lookup_type as pos_lookup_type
@@ -8,8 +8,8 @@ def feaPreamble(self, ff):
     return preamble
 
 
-def asFeaAST(self, allLanguages=[("DFLT", "dflt")]):
-    if set(allLanguages) == set(self.routine.languages):
+def asFeaAST(self, expand=False):
+    if expand or not self.routine.languages:
         if self.routine.usecount == 1:
             return self.routine.asFeaAST(inFeature=True)
         return feaast.LookupReferenceStatement(self.routine.asFeaAST())
@@ -23,6 +23,3 @@ def asFeaAST(self, allLanguages=[("DFLT", "dflt")]):
         f.statements.append(feaast.LookupReferenceStatement(self.routine.asFeaAST()))
     return f
 
-
-def asFea(self):
-    return self.asFeaAST().asFea()
